# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MasterSetting',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('display_name', models.CharField(max_length=255, verbose_name='Display name')),
                ('value', models.CharField(max_length=1000, null=True, verbose_name='Setting value')),
                ('type', models.CharField(max_length=10, verbose_name='Setting type', choices=[(b'integer', b'Integer'), (b'string', b'String'), (b'choices', b'Choices'), (b'float', b'Float'), (b'foreign', b'Foreign key')])),
                ('foreign_model', models.CharField(max_length=255, null=True, verbose_name='Foreign model')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.core.management.base import BaseCommand
from ma_settings.models import MasterSetting
from django.conf import settings
from django.db.models.loading import get_model


class Command(BaseCommand):
    args = 'No args.'
    help = 'Initializes and updates master settings.'

    def handle(self, *args, **options):
        manager = MasterSettingManager()
        manager.init_settings()


class MasterSettingManager(object):
    def init_settings(self):
        if hasattr(settings, 'MASTER_SETTINGS'):
            settings_def = settings.MASTER_SETTINGS
            for setting_name, setting_def in settings_def.items():
                if self.setting_has_changed_or_new(setting_name, setting_def):
                    self.delete_setting(setting_name)
                    self.create_new_setting(setting_def, setting_name)
                self.update_rest_fields(setting_def, setting_name)
            self.delete_old_settings(settings_def)

    def setting_has_changed_or_new(self, setting_name, setting_def):
        try:
            setting = MasterSetting.objects.get(name=setting_name)
            if setting.type != setting_def['type']:
                return True
        except MasterSetting.DoesNotExist:
            return True
        return False

    def delete_setting(self, setting_name):
        try:
            setting = MasterSetting.objects.get(name=setting_name)
            setting.delete()
        except MasterSetting.DoesNotExist:
            pass

    def create_new_setting(self, setting_def, setting_name):
        new_setting = MasterSetting(name=setting_name)
        new_setting.type = self.check_type(setting_def['type'])
        if 'display_name' in setting_def:
            new_setting.display_name = str(setting_def['display_name'])
        else:
            new_setting.display_name = str(setting_name)
        if new_setting.type == 'foreign':
            new_setting.foreign_model = self.check_model(setting_def)
        if 'default' in setting_def and new_setting.type != 'foreign':
            new_setting.value = setting_def['default']
        new_setting.save()

    def check_type(self, setting_type):
        if setting_type in ['integer', 'float', 'string', 'choices', 'foreign']:
            return setting_type
        raise Exception("Unknown setting type: %s" % setting_type)

    def check_model(self, setting_def):
        if 'model' in setting_def:
            model = setting_def['model']
            get_model(*model.split("."))
            return model
        raise Exception("Master settings: foreign setting must have 'model' field")

    def delete_old_settings(self, settings_def):
        existing_settings = MasterSetting.objects.all()
        for existing_setting in existing_settings:
            if existing_setting.name not in settings_def:
                self.delete_setting(existing_setting.name)

    def update_rest_fields(self, setting_def, setting_name):
        new_setting = MasterSetting(name=setting_name)
        if 'display_name' in setting_def:
            new_setting.display_name = str(setting_def['display_name'])
        else:
            new_setting.display_name = str(setting_name)



from django.conf import settings as settings_py
from django.db.models.loading import get_model
from django.forms import forms
from django.forms import fields
from django.forms.models import ModelChoiceField
from django.shortcuts import render
from django.views.generic import TemplateView
from ma_settings.models import MasterSetting, SettingTypes


class SettingsForm(forms.Form):
    settings = None

    def __init__(self, settings, data=None):
        super(SettingsForm, self).__init__(data=data)
        self.settings = settings
        for setting in settings:
            if setting.type == SettingTypes.STRING:
                self.fields[self.field(setting)] = fields.CharField(label=setting.display_name, initial=setting.value)
            elif setting.type == SettingTypes.INT:
                self.fields[self.field(setting)] = fields.IntegerField(label=setting.display_name, initial=setting.value)
            elif setting.type == SettingTypes.FLOAT:
                self.fields[self.field(setting)] = fields.CharField(label=setting.display_name, widget=fields.NumberInput(),
                                                                    initial=setting.value)
            elif setting.type == SettingTypes.FOREIGN:
                model = get_model(*setting.foreign_model.split("."))
                self.fields[self.field(setting)] = ModelChoiceField(label=setting.display_name, queryset=model.objects.all(),
                                                                    initial=setting.value)
            elif setting.type == SettingTypes.CHOICES:
                options = settings_py.MASTER_SETTINGS[setting.name]['options']
                choices = []
                for option in options:
                    choices.append((option, option))
                self.fields[self.field(setting)] = fields.ChoiceField(label=setting.display_name, choices=choices,
                                                                      initial=setting.value)

    def field(self, setting):
        return 'field%i' % setting.id

    def clean(self):
        for setting in self.settings:
            self.cleaned_data[self.field(setting)] = self.data[self.field(setting)]
        return self.cleaned_data

    def save(self):
        for setting in self.settings:
            value = self.cleaned_data[self.field(setting)]
            setting.value = value
            setting.save()


class HomeView(TemplateView):
    template_name = 'ma_settings/home.html'
    form = None

    def get_context_data(self, **kwargs):
        context = super(HomeView, self).get_context_data(**kwargs)
        all_settings = MasterSetting.objects.all()
        context['settings'] = all_settings
        context['base_template'] = 'ma_settings/base.html'
        context['form'] = self.form

        if hasattr(settings_py, 'BASE_SETTINGS_TEMPLATE_NAME'):
            context['base_template'] = settings_py.BASE_SETTINGS_TEMPLATE_NAME

        return context

    def dispatch(self, request, *args, **kwargs):
        all_settings = MasterSetting.objects.all()
        if request.method == 'POST':
            self.form = SettingsForm(all_settings, request.POST)
            if self.form.is_valid():
                self.form.save()
            return render(request, self.template_name, self.get_context_data(**kwargs))
        else:
            self.form = SettingsForm(all_settings)
            return render(request, self.template_name, self.get_context_data(**kwargs))